local icons = {}
local db = nil
local chardb = nil
local currentIcon = nil
local iconFrame = nil
local iconTimer = 0

local frame = CreateFrame("Frame")
frame:RegisterEvent("PLAYER_ENTERING_WORLD")
frame:SetScript("OnEvent", function(self, event, ...) self[event](self, ...) end)

function frame:PLAYER_ENTERING_WORLD()
	local function setup_db(db, defaults)
		for k,v in pairs(defaults) do
			if type(db[k]) == "nil" then
				db[k] = v
			end
		end
		return db
	end
	if not BuffDropDB then BuffDropDB = {} end
	if not BuffDropPCDB then BuffDropPCDB = {} end
	db = setup_db(BuffDropDB, {
		icon = true,
		raid = true,
		icontime = 2,
	})
	chardb = setup_db(BuffDropPCDB, {
		watch = {},
	})
	self:UnregisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
end

function frame:COMBAT_LOG_EVENT_UNFILTERED(timestamp, eventtype, sourceguid, sourcename, sourceflags, destguid, destname, destflags, ...)
	if self[eventtype] and CombatLog_Object_IsA(destflags, COMBATLOG_FILTER_MINE) then
		self[eventtype](self, ...)
	end
end

function frame:SPELL_AURA_REMOVED(spellid, spellname, spellschool, auratype)
	if chardb.watch[spellname] then
		self:AnnounceBuff(spellname)
	end
end

function frame:ENCHANT_REMOVED(enchantname, itemId, itemname)
	if chardb.watch[enchantname] then
		self:AnnounceBuff(enchantname)
	end
end

local function getBuffIcon(buff)
	for i=1, BUFF_MAX_DISPLAY do
		local n, _, t = UnitAura("player", i)
		if not n then break end
		if n == buff then return t end
	end
end

function frame:SPELL_AURA_APPLIED(spellid, spellname, spellschool, auratype)
	if chardb.watch[spellname] then
		local icon = getBuffIcon(spellname)
		if icon then icons[spellname] = icon end
	end
	if currentIcon == spellname then
		self:HideBigIcon()
	end
end
function frame:ENCHANT_APPLIED(enchantname, itemId, item)
	if chardb.watch[enchantname] then
		local _, _, _, _, _, _, _, _, _, itemTexture = GetItemInfo(itemId)
		if itemTexture then icons[enchantname] = itemTexture end
	end
	if currentIcon == enchantname then
		self:HideBigIcon()
	end
end

local color_red = {r=1, g=0, b=0}
function frame:AnnounceBuff(buff)
	if db.raid then
		RaidNotice_AddMessage(RaidWarningFrame, buff, color_red)
	end
	if db.icon then
		currentIcon = buff
		self:ShowBigIcon(icons[buff])
	end
end

local function onUpdate(self, elapsed)
	iconTimer = iconTimer + elapsed
	iconFrame:SetVertexColor(1, 1, 1, 0.5 - (0.5 * (iconTimer / db.icontime)))
	if iconTimer > db.icontime then
		frame:HideBigIcon()
	end
end

function frame:ShowBigIcon(icon)
	if not iconFrame then
		frame:SetFrameStrata("HIGH")
		frame:SetPoint("CENTER", WorldFrame)
		local width = WorldFrame:GetHeight() / 3
		frame:SetWidth(width)
		frame:SetHeight(width)

		iconFrame = frame:CreateTexture()
		iconFrame:SetAllPoints(frame)
		iconFrame:SetBlendMode("BLEND")
		iconFrame:SetTexCoord(0.09, 0.91, 0.09, 0.91)

		frame:SetScript("OnUpdate", onUpdate)
		frame:Hide()
	end
	iconFrame:SetVertexColor(1, 1, 1, 0.5)
	iconFrame:SetTexture(icon)
	self:Show()
end
function frame:HideBigIcon()
	currentIcon = nil
	iconFrame:SetTexture(1, 0, 0)
	self:Hide()
	iconTimer = 0
end

local function announce(message, r, g, b)
	DEFAULT_CHAT_FRAME:AddMessage(message, r, g, b)
end
local menu_options = {
	icon = function(arg)
		if not arg then return " - big icon (" .. (db.icon and "true" or "false") .. ")" end
		db.icon = not db.icon
		announce("Huge icon "..(db.icon and "enabled" or "disabled"), 1, 1, 1)
	end,
	raid = function(arg)
		if not arg then return " - raid warning (" .. (db.raid and "true" or "false") .. ")" end
		db.raid = not db.raid
		announce("Raid warning "..(db.raid and "enabled" or "disabled"), 1, 1, 1)
	end,
	watch = function(arg)
		if not arg then return " - buffs to watch" end
		if arg == "" then
			announce("Watching:", 1, 1, 1)
			for buff in pairs(chardb.watch) do
				announce(" "..buff, 1, 1, 1)
			end
			return
		end
		if chardb.watch[arg] then
			chardb.watch[arg] = nil
			announce("No longer watching "..arg, 1, 1, 1)
		else
			chardb.watch[arg] = true
			announce("Now watching "..arg, 1, 1, 1)
		end
	end,
	icontime = function(arg)
		if not arg then return " - icon lingers for (" .. db.icontime .. ")" end
		local newtime = tonumber(arg)
		if newtime then
			db.icontime = newtime
		end
	end,
}
SLASH_BUFFDROP1 = "/buffdrop"
SLASH_BUFFDROP2 = "/bd"
SlashCmdList["BUFFDROP"] = function(arg)
	local command, arguments = string.match(arg, "^(%a+) ?(.*)")
	if menu_options[command] then
		return menu_options[command](arguments)
	end
	announce("BuffDrop:", 1, 1, 1)
	for k,v in pairs(menu_options) do
		announce(" -"..k..v(), 1, 1, 1)
	end
end

